function E = hameig(A,G,Q,bal);
%HAMEIG    Eigenvalues of Hamiltonian matrices.
%   E = HAMEIG(A,G,Q) is a vector containing the eigenvalues of a Hamiltonian
%   matrix H defined as
%   
%         H = [A  G; Q  -A'],
%
%   where  G = G',  Q = Q'.
%
%   E = HAMEIG(A,G,Q,'nobalance') performs the computation with balancing
%   disabled, which sometimes gives more accurate results for certain
%   problems with unusual scaling.
%
%   This function uses Van Loan's square-reduced method with balancing as
%   implemented in the SLICOT subroutines MB04ZD and MB04DY, respectively. 
%   The corresponding SLICOT routines are called via gateways produced by
%   the NAG Gateway generator.  

%   Peter Benner,  2-19-1999.
%   Copyright held by the author.

%
error(nargchk(3,4,nargin))
%
if (nargin == 4) & (bal == 'nobalance'),
  jobscl = 'N';  
else 
  bal    = 'B';
  jobscl = 'S';
end
[na,ma] = size(A);
if (na ~= ma),  
  error('Input matrix A must be square.'),  
end
[ng,mg] = size(G);
if (ng ~= na) | (ng ~= mg),  
  error('Input matrix G has incompatible dimensions.'),  
end
[nq,mq] = size(Q);
if (nq ~= na) | (nq ~= mq),  
  error('Input matrix Q has incompatible dimensions.'),  
end
n = na;
if norm(G-G',1)/norm(G,1) > n*eps,
  error('Input matrix G must be symmetric.')
end
if norm(Q-Q',1)/norm(Q,1) > n*eps,
  error('Input matrix Q must be symmetric.')
end
%
QG = [tril(Q) zeros(n,1)] + [zeros(n,1) triu(G)];
% call SLICOT gateways
if (jobscl ~= 'N')
  [A,QG] = mb04dy(A,QG,jobscl);
end 
[A,QG] = mb04zd(A,QG);
if (bal == 'nobalance')
  [wr,wi,info] = mb03sd(A,QG,'N');
else
  [wr,wi,info] = mb03sd(A,QG,'B');
end
if (info ~= 0),
  error('Could not compute eigenvalues, error no. %d', info)
else
  E = wr + sqrt(-1)*wi;
  E = [E; -E]; 
end
